function [data, A] = conj_coef(data, A, average)

% CZ
% Tato funkce dopotv komplexn sdruen koeficienty Fourierovy transformace, kter nebyly nameny pi MRI (byl namen 
% pouze jeden z tchto koeficient). Ppadn tato funkce prmruje hodnoty, kter maj bt komplexn sdruen, ale nejsou kvli umu.
% Vstupn parametry jsou:
%     data ... namen hodnoty videa z MRI ve frekvenn oblasti, v kadm sloupci matice dat je jeden snmek
%     A ... mic matice, pomoc kter byly nameny data (zskna nap. pomoc funkce generate_mask)
%     (average) ... nepovinn parametr, kter je bu true nebo false, explicitn false 
%                       jestlie zadme true, zprmruj se ty koeficienty, kter maj bt komplexn sdruen
% Vstupem jsou dopotan vstupn parametry.
% Pouit: [data, A] = conj_coef(data, A)
%             [data, A] = conj_coef(data, A, average)
% -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% EN
% This function computes Fourier transform conjugate coefficient which were not measured by MRI (it was measured only one of this 
% coefficient). In addition this function can average coefficient which should be conjugated but they are not because of noise.
% Input parameters are:
%     data ... measured video data from MRI in frequency plane, in each column of this matrix is one frame
%     A ... measure matrix, data was measured by this matrix (e.g. from function generate_mask)
%     (average) ... optional parameter true or false, default false 
%                       in case of true coefficients which should be conjugated are averaged
% Output of this function are completed input parameters.
% Use: [data, A] = conj_coef(data, A) 
%         [data, A] = conj_coef(data, A, average)

    % nastaven nepovinnho parametru / setting up optional parameter
    if nargin < 3, average = false; end 
    
    n = sqrt(size(data,1));
    t = size(data,2);
    data = reshape(data, n, n, t);
    A = reshape(A, n, n, t);
 
    pocet01 = 0;
    pocet00 = 0;
     
    % dopotn komplexn sdruench koeficient, kter nebyly nameny 
    %/ computation of conjugate coefficient which were not measured
    for i = 1:t
         [r, s] = find(1-A(:,:,i)); 
         for j = 1: length(r)
             if r(j) == 1
                 if s(j) ~= 1
                     if A(1, n + 2 - s(j), i) == 1
                        A(1, s(j), i) = 1;
                        data(1, s(j), i) = conj(data(1, n + 2 - s(j), i));
                        pocet01 = pocet01 + 1;
                     else
                         if s(j) ~= n/2+1
                            pocet00 = pocet00 + 1;
                         end
                     end
                 end
             else
                 if s(j) == 1
                     if  A(n + 2 - r(j), 1, i) == 1
                        A(r(j), 1, i) = 1;
                        data(r(j), 1, i) =  conj(data(n + 2 - r(j), 1, i));
                        pocet01 = pocet01 + 1;
                     else
                         if r(j) ~= n/2+1
                            pocet00 = pocet00 + 1;
                         end
                     end
                 else
                     if A(n + 2 - r(j), n + 2 -s(j), i) == 1
                        A(r(j), s(j), i) =  1;
                        data(r(j), s(j), i) =  conj(data(n + 2 - r(j), n + 2 -s(j), i));
                        pocet01 = pocet01 + 1;
                     else
                         if r(j) == n/2 + 1  
                             if s(j) == n/2 + 1
                             else
                                pocet00 = pocet00 + 1;
                             end
                         else
                            pocet00 = pocet00 + 1;
                         end
                     end
                 end
             end
         end
    end

    % prmrovn dvojic koeficient, kter maj bt komplexn sdruen
    % / averaging pairs of coefficients which have to be copml. conjugated 
    if average
        for i = 1:t
            b = data(:,:,i);
            for j = 2:n/2
                b(1,j) = (b(1,j) + conj(b(1,n+2-j)))/2;
                b(1,n+2-j) = conj(b(1,j));
                b(j,1) = (b(j,1) + conj(b(n+2-j,1)))/2;
                b(n+2-j,1) = conj(b(j,1));
            end

            for j = 2:n
                for k = 2:n/2+1
                    b(j,k) = (b(j,k) + conj(b(n+2-j,n+2-k)))/2;
                    b(n+2-j,n+2-k) = conj(b(j,k));
                end
            end
            data(:,:,i) = b;
        end
    end
    
    data = reshape(data, n^2, t);
    A = reshape(A, n^2, t);
    
    % dopotn typu koeficient / computation of coefficient type
    pocet00 = pocet00/2;
    if mod(n,2) == 0
        pocet = 4*t;
    else
        pocet = t;
    end
    pocet11 = (n*n*t - pocet - 2*pocet01 - 2*pocet00)/2;
    
    display(['neprovch koeficient / not paired coeficients: ', num2str(pocet)])
    display(['pr typu / pair of type 1-1: ', num2str(pocet11)]) 
    display(['pr typu / pair of type 1-0: ', num2str(pocet01)])
    display(['pr typu / pair of type 0-0: ', num2str(pocet00)])
end