% CZ
% Tento program pedvd, jak z namenho videa perfznho zobrazovn pomoc MRI zptn rekonstruovat data, a to jak 
% simulovan, tak i reln.
% -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% EN
% This program shows how data from measured perfusion MRI video can be reconstructed. Data can be simulated or real.  

clear; clc; close all;
tic

%% nastaven parametr / setting up parameters

% typ vstupnch dat / type of input data
% data = 'real';
data = 'simulated';

% nastaven parametr pro simulovan data / setting up parameters to simulated data
if strcmp(data,'simulated')
    n = 150; % rozmr tvercovho snmku / size of squared frame
    
    % parametry pro lognormln model / parameters to lognormal model
    t0 =1; % posunut lognormlnho rozdlen / shift of lognormal distribution
    % parametry lognormlnho rozdlen  (2x) / parameters of  lognormal distribution (2x)
    mi = 5.5;
    sigma = 0.7;    
    mi2 = 5;
    sigma2 = 0.8;
    S = 8.6*10^6; % plocha pod grafem (stejn pro ob hustoty) / area under curve (same for both)
    % pedpokld se, e doba sledovn a frekvence jsou soudln / it is assumed that duration of observation and frequency are divisible
    T = 600; % doba sledovn / duration of observation
    f = 1/6; % frekvence snmn hodnot / frequency of scanning
end

param = struct; % vytvoen nepovinnho parametru / creation of optional parameter

% typ masky (trajektorie) ve frekvennm prostoru (a nastaven jejch parametr)                                 
% / type of the mask (trajectory) in the k-space (and setting up their parameters)

mask_type = 'm'; % nhodn nerovnomrn / random not uniform
param.pst = 0.5;
% param.ctverec = true;

% mask_type = 'r'; % radiln / radial
% param.pocet = 60;

% mask_type = 'n'; % nhodn / random
% param.probability = [0.25 0.5]; % pravdpodobnost namen dat / probability of data measurement, 0 < param.probability < 1
% param.conj = [true false]; % true => komplexn hodnoty nebudou nameny / complex conjugates are not subject to measurement

% mask_type = 's'; % spirln / spiral
% param.number = 40;
% param.p = 5;
% param.a = 1;
% param.b = 0.5;

% mask_type = 'sr'; % stdn masek / alternation of masks
% param.p = 1000;

% pesnost rekonstrukce (nejvt ppustn relativn zmna aproximace) 
% / precision of reconstruction (maximal feasible relative change of approximation)
epsilon = 10^(-5);  

% pidn blho umu k datm / adding of aditive white noise to data
bily_sum = false; 
% bily_sum = true; 

% smrodatn ochylka umu / standard deviation of noise
if bily_sum
    odchylka = 0.01; 
end


%% vytvoen (naten) dat / creation (reading) of data

switch data
    
    % simulovan data / simulated data
    case 'simulated'
    
        % nastaven parametr / setting up parameters
        m = n;
        t = T*f; % poet snmk / number of frames

        % vytvoen perfznho modelu / creation of perfusion model
        perfuse.data1 = lognormal_model(t0, mi, sigma, S, T, f);
        perfuse.data2 = lognormal_model(t0, mi2, sigma2, S, T, f);
    
        % vytvoen simulovanch dat / creation of simulated data
        original = phantom_video(n, perfuse);

    % reln data / real data
    case 'real'
        mys = load('MGEmultiFA_sec01_inp_130328'); % naten dat / reading of data
        t = length(mys.data2);
        [m n] = size(mys.data2{1});
        original = zeros(m,n,t); % peskldn dat do trojrozmrn matice / creating 3D data matrix
        for i = 1:t
            original(:,:,i) = mys.data2{i};
        end
end


%% namen dat / data measurement

% vytvoen masky / creation of mask
A = generate_mask(n, t, mask_type,param);  

% uloen Fourierovy transformace jednotlivch snmk do sloupc 2D matice 
% / storing Fourier transforms of individual videoframes as columns of a 2D matrix
fourierorig = zeros(n^2,t);

for i = 1:t
    if strcmp(data, 'real')
        fourier = zeros(n,n); % pprava na dopoet na tvercov snmky / preparation to computation (frames are assumed to be squared)
        fourierdat = fft2(original(:,:,i)); 
        fourier(1:m/2,:) = fourierdat(1:m/2,:);
        fourier((m/2 + n - m +1):n,:) = fourierdat((m/2+1):m,:);
    else
         fourier = fft2(original(:,:,i)); 
    end
    fourierorig(:,i) = reshape(fourier,[],1);
end

% dopoet masky kvli tvercovm snmkm (my nen tvercov) / computation of mask due to squared frames (mouse is not square)
 if strcmp(data, 'real')
     for i = 0:n-1
        A(n*i+(m/2+1):n*i+(m/2 + n - m),:) = 0; 
     end
 end

% pidn umu / adding moise
if bily_sum
    fourierorig = fourierorig + odchylka*randn(size(fourierorig));
end

% peskldn matice originalnho videa do 2D matice (snmek jako sloupec) / reshape original video into 2D matrix (frames as columns)
original = reshape(original,m*n,t);

% namen dat / data measurement 
d = A.*fourierorig;

% dopotn komplexn sdruench koeficient / computation of conjugate coeficient
if bily_sum
     % + zprmrovn koeficient, kter nejsou pesn komplexn sdruen 
     % / + average of coefficients which are not completely conjugated
    average = true; 
    [d, A] = conj_coef(d, A, average); 
else
    [d, A] = conj_coef(d, A);
end


%% rekonstrukce dat / reconstruction of data

[M,L,S] = LS_decomposition(d,A,epsilon);


%% vykreslen vsledk / result depiction 

% hledn mtka grafu / searching for graph scale
a = max(max(original));
b = max(max(M));
c = max(max(L));
d = max(max(S));
maximum = max([a b c d]);

a = min(min(original));
b = min(min(M));
c = min(min(L));
d = min(min(S));
minimum = min([a b c d]);

% vykreslen vsledk / result depiction 
figure
imagesc(original, [minimum maximum])
title('originl / original')
colorbar

figure
imagesc(M, [minimum maximum])
title('M (rekonstrukce / reconstruction, M = L+S)')
colorbar

figure
imagesc(L, [minimum maximum])
title('L (matice s nzkou hodnost / low rank matrix)')
colorbar

figure
imagesc(S, [minimum maximum])
title('S (matice dk ve spektru po dcch / sparse matrix in row frequency domain)')
colorbar

if strcmp(data, 'simulated')
    disp(['relativn chyba rekonstrukce / relative error of reconstruction ', num2str(norm(M - original)/norm(original))])
    disp(['SNR ', num2str( 10*log10( mean(original.^2)/mean( (M - original).^2 ) ) )])
end

%% uloen vsledk / result saving

M = reshape(M,n,n,t);
export_avi(M, 'rekonstrukce') 

toc