function maska = generate_mask(n, t, typ, param)

% CZ
% Tato funkce vytv mic matici (masku) na men videa. Matice pedpokld, e jednotliv snmky videa jsou ve frekvenn oblasti 
% a jsou naskldny ve sloupcch 2D matice. 
% Vstupn parametry jsou:
%     n ... rozmr jednotlivch tvercovch snmk
%     t ... poet snmk
%     typ ... druh masky: 'm' (nerovnomrn nhodn - vy pst v potku), 'r' (radiln ry v jednotlivch snmcch), 'n' (nhodn) nebo 
%               's' (spirly v jednotlivch snmcch)
%               jednotliv druhy mohou bt kombinovny  (nap. 'rns') - masky se stdaj podle danho schmatu
%     (param) ... nepovinn parametr, zle na druhu masky (me bt i vektor, parametry se potom stdaj): 
%                     'm' - param.pst ... piblin procento namench vzork, defaultn 0.5 
%                          - param.ctverec ...  tvercov vrstevnice pravdpodobnosti, defaultn false
%                     'r' - param.pocet ... poet pmek, defaultn round(6*n/10)
%                     'n' - param.probability ... pravdpodobnost namen koeficientu, 0 < param < 1 (pi param.conj == true m smysl 
%                                                         pouze do 0.5), defaultn 0.5 
%                         - param.conj ... zamezen vbru komplexn sdruench koeficient (pi hodnot true), defautn false     
%                     's' - param.number ... poet spirl, defaultnround(4*n/10)
%                         - param.p ... parametr "tvercovatn" spirly pomoc p normy, 1 <= param.p < infty, defaultn 2
%                         - param.a ... vzdlenost potku spirly od jejho plu, defaultn nhodn rand + 0.5
%                         - param.b ... nrst polomru (hel tenho vektoru) spirly, defaultn nhodn rand + 0.1
%  Pouit: maska = generate_mask(n, t, typ)
%              maska = generate_mask(n, t, typ, param)                 
% ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% EN
% This function creates measuring matrix (mask) to measure video. It is assumed that each frame is in frequency domain and it is in 
% one column of 2D matrix.
% Input parameters are:
%     n ... size of square frames
%     t ... number of frames
%     typ ... kind of mask: 'nrand' (random but not uniformly distributed - higher density in origin), 'r' (radial lines in each frame), 'n' (random) or 
%               's' (spirals in each frame)
%               masks can be combinated (e.g. 'rns') - masks are alternated according to the scheme
%     (param) ... optional parameter depends on kind of mask (it can be vector - parameters are alternated):
%                     'm' - param.pst ... rough percent of measured data, defaut 0.5
%                          -  param.ctverec ... square contour line of probability, default false
%                     'r' - param.pocet ... number of lines, default round(6*n/10)
%                     'n' - param.probability ...  probability of data measurement, 0 < param < 1 (in case of param. conj == true it has 
%                                                          sence only to value 0.5), default 0.5 
%                         - param.conj ... avoid choosing conjugate coefficients (in case true), defaut false     
%                     's' - param.number ... number of spirals, default round(4*n/10)
%                         - param.p ... parameter of "squaring" spiral using p norm, 1 <= param.p < infty, default 2
%                         - param.a ... distance between beginning of spiral and its pole, default randomly rand + 0.5
%                         - param.b ... increase the radius (angle of the tangent vector), default randomly rand + 0.1
% Use: maska = generate_mask(n, t, typ)
%         maska = generate_mask(n, t, typ, param)       

    % nastaven nepovinnch parametr / setting up optional parameters
    if nargin < 4, param = struct;end
    parametr = struct;
    
    maska = zeros(n^2,t);
    delka = length(typ);
    pocetm = 0;
    pocetr = 0;
    pocetn = 0;
    pocets = 0;
    
    for i = 1:t

        switch typ( mod(i,delka) + ( mod(i,delka) == 0 )*delka )
        
            % nhodn maska s nerovnomrnm rozdlenm (vt pravdpodobnost smrem k potku k-prostoru) 
            %/ random mask with not uniform distribution (bigger density in the coordinate origin of k-space)
            case 'm'
                pocetm = pocetm + 1;
                
                if isfield(param, 'pst') 
                    d = length(param.pst);
                    parametr.pst = param.pst( mod(pocetm,d) + ( mod(pocetm,d) == 0 )*d ); 
                end
                
                 if isfield(param, 'ctverec') 
                    d = length(param.ctverec);
                    parametr.ctverec = param.ctverec( mod(pocetm,d) + ( mod(pocetm,d) == 0 )*d ); 
                end
                
                matice = mask(n,'nrand',parametr);
                matice = ifftshift(matice);
            
            % radiln maska / radial mask
            case 'r'
                pocetr = pocetr + 1;
                
                if isfield(param, 'pocet') 
                    d = length(param.pocet);
                    parametr.pocet = param.pocet( mod(pocetr,d) + ( mod(pocetr,d) == 0 )*d ); 
                end
                
                matice = mask(n,'radial',parametr);
                matice = ifftshift(matice);

            % nhodn maska / random mask
            case 'n'
                pocetn = pocetn + 1;
                
                if isfield(param, 'probability') 
                    d = length(param.probability);
                    parametr.probability = param.probability( mod(pocetn,d) + ( mod(pocetn,d) == 0 )*d ); 
                end
        
                if isfield(param, 'conj') 
                    d = length(param.conj);
                    parametr.conj = param.conj( mod(pocetn,d) + ( mod(pocetn,d) == 0 )*d ); 
                end
       
                matice = mask(n,'random',parametr);

            % spirln maska / spiral mask
            case 's'
                pocets = pocets + 1;
                
                if isfield(param, 'number') 
                    d = length(param.number);
                    parametr.number = param.number( mod(pocets,d) + ( mod(pocets,d) == 0 )*d ); 
                end
         
                if isfield(param, 'p') 
                    d = length(param.p);
                    parametr.p = param.p( mod(pocets,d) + ( mod(pocets,d) == 0 )*d ); 
                end
         
                if isfield(param, 'a') 
                    d = length(param.a);
                    parametr.a = param.a( mod(pocets,d) + ( mod(pocets,d) == 0 )*d ); 
                end
         
                if isfield(param, 'b') 
                    d = length(param.b);
                    parametr.b = param.b( mod(pocets,d) + ( mod(pocets,d) == 0 )*d ); 
                end
                
                matice = mask(n,'spiral',parametr);
                matice = ifftshift(matice);
                
        end
        
        matice = matice(:);
        maska(:,i) = matice;
    end

     pocet1 = sum(sum(maska)); %number of ones in the masks, i.e. number of measured coefs
     display(['Zmench koeficient je ', num2str(pocet1), ' z ', num2str(n^2*t), ', co je ', num2str(pocet1/(n^2*t)*100),' %'])
     display(['Number of measured coefficients is ', num2str(pocet1), ' out of ', num2str(n^2*t), ', which means ', num2str(pocet1/(n^2*t)*100),' %'])
end
