 function fx = lognormal_model(t0, mi, sigma, S, T, f, spectrum)

% CZ
% Tato funkce vytv model prbhu bolusu pi perfznm zobrazovn v magnetick rezonanci. Prbh je modelovn jako funkce 
% hustoty pravdpodobnosti lognormlnho rozdlen s tmito vstupnmi parametry:
%     t0 ... posunut lognormlnho rozdlen
%     mi, sigma ... parametry lognormlnho rozdlen
%     S ... plocha pod grafem hustoty lognormlnho rozdlen (jednotkov plocha se vynsob touto konstantou)
%     T ... celkov as, po kter sledujeme toto rozdlen
%     f ... frekvence snmn hodnot (pedpokld se, e T a f jsou soudln)
%     (spectrum) ... nepovinn parametr - pi hodnot true vykresl absolutn hodnotu spektra lognormlnho modelu, defaultn false
% Pouit: fx = lognormal_model(t0, mi, sigma, S, T, f)
%              fx = lognormal_model(t0, mi, sigma, S, T, f, spectrum)
% ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ 
% EN
% This function creates an indicator dilution model for bolus of MRI contrast agent. This model uses function of probability density of 
% lognormal distribution with these input parameters: 
%     t0 ... shift of lognormal distribution
%     mi, sigma ... parameters of  lognormal distribution
%     S ... area under curve  (unit area is multiplied by this constant)
%     T ... time period for which we observe this distribution
%     f ... frequency of scanning (it is assumed that T and f are divisible)
%     (spectrum) ... optional parameter - in case of true this function plots absolut value of spectrum of lognormal model, defaut false
% Use: fx = lognormal_model(t0, mi, sigma, S, T, f)
%          fx = lognormal_model(t0, mi, sigma, S, T, f, spectrum)

    % nastaven nepovinnch parametr / setting up optional parameters
    if nargin < 7, spectrum = false; end 
    
    t = linspace(f,T,f*T);
    pocet = length(t);
    fx = zeros(1,pocet);
    
    % vpoet lognormlnch dat / computation of lognormal data
    for i = 1:pocet
        if t(i) > t0
             fx(i) = S* exp( - (log(t(i) - t0) -mi)^2/( 2*sigma^2) )/( (t(i) - t0)*sigma*sqrt(2*pi) );
        end
    end
    
    % vykreslen spektra / depiction of spectrum
    if spectrum
        figure
        plot(abs(fftshift(fft(fx))))
    end
    
 end