function [mask, scale, max_mean, average] = nonuniform_random_mask(rel_density, mean)

% rel_density........relative density of probability, has to be >=0, one or two-dimensional
% mean.......expected ratio of number of ones in the mask to full size of mask
% mask......generated mask (ones/zeros), the same size as rel_density
% scale........scaling factor used to generate the resulting mask

s = size(rel_density);
elements = prod(s);
sum_relative = sum(sum(rel_density));

%multiplication of rel_density by scale_full leads to "full range" density
scale_full = 1/ max(max(rel_density));

%determine maximum possible mean (in expectation)
max_mean = scale_full * sum_relative / elements;

if max_mean < mean && abs(max_mean - mean) > 0.00001
  disp('Warning: Parametr ''mean'' je zvolen prilis vysoko vzhledem k defiovane ''rel_density''.')
  disp('Aby bylo dosaeno tohoto ''mean'', dojde nyn ke zmn pomr pravdpodobnost v masce.')
  %
  disp('Warning: Such high ''mean'' in not possible (in expectation) regarding ''rel_density''.')
  disp('To achieve this ''mean'', the relative density will be modified.')

  pom = true;
  rel_density = rel_density * scale_full;
  while pom
    sum_relative = sum(sum(rel_density));
    q =  mean - sum_relative / elements;
    rel_density = min(rel_density+q,1);
    if q < 0.0001
        pom = false;
    end
  end
  scale = 'nedefinovana';
else
    %determine proper scale
    scale = mean * elements / sum_relative;
    rel_density = scale * rel_density;
end

mask_uniform = rand(s);
mask = (mask_uniform < rel_density  );

average = sum(sum(mask)) / elements;
