function video = phantom_video(n, perfuse, dim)

% CZ
% Tato funkce vytv model perfznho zobrazovn mozku v jednom ezu. Vstupn parametry jsou:
%     n ... rozmr jednotlivch tvercovch snmk
%     perfuse ... skld se z perfuse.data1 a perfuse.data2, co jsou hodnoty zvislosti prbhu bolusu pi perfznm zobrazovn
%                     (zskny nap. pomoc funkce lognormal_model)
%     (dim) ... nepovinn parametr udvajc dimenzi snmn - bu '2d' nebo '3d'
% Vstupem je trojrozmrn matice dat.
% Pouit: video = phantom_video(n, perfuse)
%             video = phantom_video(n, perfuse, dim)
% -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% EN
% This function creates model of perfusion imaging of brain in one slice. Input parameters are:
%     n ... size of square frames
%     perfuse ... consists of perfuse.data1 and perfuse.data2, which are values of perfusion data 
%                     (e.g. from function lognormal_model)
%     (dim) ... optional parameter represents dimension of scanning - can be '2d' or '3d'
% Output is 3D matrix of video data.
% Use: video = phantom_video(n, perfuse)    
%         video = phantom_video(n, perfuse, dim)

    if nargin < 3, dim = '2d'; end
    
    fx = perfuse.data1;
    fx2 = perfuse.data2;
    pocet = length(fx);
    
    % normovn perfznch dat / normalization of perfusion data
    % konstanty jsou zvoleny tak, aby nedolo k saturaci dat / constants are chosen so as to avoid saturation of data 
    maximum = max(fx);
    fx = fx/maximum*0.8; 
    maximum = max(fx2);
    fx2 = fx2/maximum*0.9;
    
    % poten nastaven / initialization
    switch dim
        case '2d'
            [p E] = phantom; % zskn matice parametr / acquisition of parameter matrix
            video = zeros(n,n,pocet);
        case '3d'
            [p E] = phantom3d; % zskn matice parametr / acquisition of parameter matrix
            video = zeros(n,n,n,pocet);
    end
    E(2,1) = -1;
    E(3:4,1) = 0.2;
    e =  E(3:4,1);
    f = E(5,1);
    
    % vytvoen videa / video creation
    for i = 1:pocet
        switch dim
             case '2d'
                video(:,:,i) = phantom(E,n);
             case '3d'
                video(:,:,:,i) = phantom3d(E,n);
        end
        E(3:4,1) = e + fx(i);
        E(5,1) = f + fx2(i);
    end
    
end