clear; clc; close all;
tic
%% nastaven parametr / setting up parameters


% nastaven parametr pro simulovan data / setting up parameters to simulated data
n = 150; % rozmr tvercovho snmku / size of squared frame
   
% parametry pro lognormln model / parameters to lognormal model
t0 =1; % posunut lognormlnho rozdlen / shift of lognormal distribution
% parametry lognormlnho rozdlen  (2x) / parameters of  lognormal distribution (2x)
mi = 5.5;
sigma = 0.7;    
mi2 = 5;
sigma2 = 0.8;
S = 8.6*10^6; % plocha pod grafem (stejn pro ob hustoty) / area under curve (same for both)
% pedpokld se, e doba sledovn a frekvence jsou soudln / it is assumed that duration of observation and frequency are divisible
T = 600; % doba sledovn / duration of observation
f = 1/6; % frekvence snmn hodnot / frequency of scanning


param = struct; % vytvoen nepovinnho parametru / creation of optional parameter

graf = zeros(100,8);


% pesnost rekonstrukce (nejvt ppustn relativn zmna aproximace) 
% / precision of reconstruction (maximal feasible relative change of approximation)
epsilon = 10^(-5);  


%% vytvoen  dat / creation of data


% nastaven parametr / setting up parameters
m = n;
t = T*f; % poet snmk / number of frames

% vytvoen perfznho modelu / creation of perfusion model
perfuse.data1 = lognormal_model(t0, mi, sigma, S, T, f);
perfuse.data2 = lognormal_model(t0, mi2, sigma2, S, T, f);
    
% vytvoen simulovanch dat / creation of simulated data
original = phantom_video(n, perfuse);

for k = 3
    iter = 0; % potadlo 
    switch k
        case 1
            mask_type = 'm'; % nhodn nerovnomrn / random not uniform
            par = 0:0.005:1.1;
        case 2
            mask_type = 'n';
            par = 0.05:0.005:1;
        case 3
            mask_type = 'r';
            par = [5:1:119 120:5:200 210:10:500];
        case 4
            mask_type = 's';
            par = [5:1:79 80:2:100 105:5:200 210:10:300 350 400];
    end
for j = 1:length(par)
%% namen dat / data measurement

switch k
    case 1
        param.pst = par(j)
    case 2
        param.probability = par(j)
    case 3
        param.pocet = par(j)
    case 4
        param.number = par(j)
end

% vytvoen masky / creation of mask
A = generate_mask(n, t, mask_type,param);  

pocet1 = sum(sum(A))/(n^2*t)*100; 
if j~=1
    if pocet1 - graf(iter,2*k-1) < 1
        continue;
    else
        iter = iter + 1;
    end
else
    iter = iter + 1;
end

graf(iter,2*k-1) = pocet1;

% uloen Fourierovy transformace jednotlivch snmk do sloupc 2D matice 
% / storing Fourier transforms of individual videoframes as columns of a 2D matrix
fourierorig = zeros(n^2,t);

for i = 1:t
    fourier = fft2(original(:,:,i)); 
    fourierorig(:,i) = reshape(fourier,[],1);
end

% peskldn matice originalnho videa do 2D matice (snmek jako sloupec) / reshape original video into 2D matrix (frames as columns)
original = reshape(original,m*n,t);

% namen dat / data measurement 
d = A.*fourierorig;

[d, A] = conj_coef(d, A);


%% rekonstrukce dat / reconstruction of data

[M,L,S] = LS_decomposition(d,A,epsilon);

disp(['relativn chyba rekonstrukce / relative error of reconstruction ', num2str(norm(M - original)/norm(original))])
disp(['SNR ', num2str( 10*log10( mean(original.^2)/mean( (M - original).^2 ) ) )])
graf(iter,2*k) = 10*log10( mean(original.^2)/mean( (M - original).^2 ) ); 

original = reshape(original,m,n,t);

end
end
%% uloen vsledk / result saving

save('graf_SNR.mat','graf')
toc